﻿using System;
using System.Windows.Forms;

/*
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
*/

namespace Win_OS_Forms_Application.Object
    {
    public static class ControlExtensions
        {
        /// <summary>
        /// Executes the Action asynchronously on the UI thread, does not block execution on the calling thread.
        /// No action will be performed if the control doesn't have a valid handle or the control is Disposed/Disposing.
        /// </summary>
        /// <param name="control">the control for which the update is required</param>
        /// <param name="action">action to be performed on the control</param>
        public static void InvokeOnUiThreadIfRequired(this Control control, Action action)
            {
            //See https://stackoverflow.com/questions/1874728/avoid-calling-invoke-when-the-control-is-disposed
            //for background and some guidance when implementing your own version.
            //No action
            if (control.Disposing || control.IsDisposed || !control.IsHandleCreated)
                {
                return;
                }

            if (control.InvokeRequired)
                {
                control.BeginInvoke((Action)(() =>
                {
                    //No action
                    if (control.Disposing || control.IsDisposed || !control.IsHandleCreated)
                        {
                        return;
                        }

                    action();
                }));
                }
            else
                {
                action.Invoke();
                }
            }
        }
    }
